package pressure

import (
	"context"
	"log"
	"os"

	"github.com/go-flucky/flucky/pkg/types"

	"github.com/go-flucky/flucky/pkg/cli"
	"github.com/go-flucky/flucky/pkg/config"
	"github.com/go-flucky/flucky/pkg/logfile"
	"github.com/go-flucky/flucky/pkg/rgbled"
	"github.com/go-flucky/flucky/pkg/sensor"
	"github.com/spf13/cobra"
)

var logs bool

var readPressureCmd = &cobra.Command{
	Use:   "read",
	Short: "Operates with air pressure values",
	Run: func(cmd *cobra.Command, args []string) {

		// read configuration
		cnf, err := config.Read(*configFile)
		if err != nil {
			log.Fatalln(err)
		}

		// fetch all temperature sensors or sensors by args
		sensors := make([]sensor.Sensor, 0)
		if len(args) == 0 {
			sensors = cnf.GetPressureSensors(config.ENABLED)
		} else {
			sensors = cnf.GetPressureSensorsByName(args)
		}

		if len(sensors) == 0 {
			return
		}

		rgbLEDs := cnf.GetRGBLEDs(config.ENABLED)
		if err := rgbled.Run(rgbLEDs); err != nil {
			log.Fatalln(err)
		}

		ctx := context.Background()
		measuredValues, err := sensor.Read(ctx, sensors)
		if err != nil {
			log.Fatalln(err)
		}

		measuredValues = types.SelectMeasuredValues(types.MeasuredValueTypePressure, measuredValues)

		cli.PrintMeasuredValues(measuredValues, cnf, os.Stdout)

		if logs {
			measuredValuesLogfile := logfile.New(cnf.Device.Logfile)
			err := logfile.Append(measuredValuesLogfile, compression, round, measuredValues)
			if err != nil {
				log.Fatalln(err)
			}
		}

		rgbled.Off(rgbLEDs)
	},
}

func init() {
	pressureCmd.AddCommand(readPressureCmd)
	readPressureCmd.Flags().BoolVar(&logs, "logs", true, "Log temperature")
	readPressureCmd.Flags().BoolVar(&compression, "compression", true, "Compress measured with logged temperatures")
	readPressureCmd.Flags().Float64VarP(&round, "round", "r", 0.25, "Round values. The value 0 deactivates the function")
}
